package org.gcube.portlets.user.speciesdiscovery.server.util;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.antlr.runtime.ANTLRStringStream;
import org.antlr.runtime.CommonTokenStream;
import org.antlr.runtime.RecognitionException;
import org.apache.log4j.Logger;
import org.gcube.dataaccess.spql.SPQLLexer;
import org.gcube.dataaccess.spql.SPQLParser;
import org.gcube.dataaccess.spql.model.ParsingResult;
import org.gcube.dataaccess.spql.model.ReturnType;
import org.gcube.dataaccess.spql.model.TermType;
import org.gcube.portlets.user.speciesdiscovery.shared.SearchByQueryParameter;
import org.gcube.portlets.user.speciesdiscovery.shared.SearchFilters;
import org.gcube.portlets.user.speciesdiscovery.shared.SearchResultType;
import org.gcube.portlets.user.speciesdiscovery.shared.SearchServiceException;
import org.gcube.portlets.user.speciesdiscovery.shared.SearchType;
import org.gcube.portlets.user.speciesdiscovery.shared.SpeciesCapability;

/**
 * 
 * @author Francesco Mangiacrapa francesco.mangiacrapa@isti.cnr.it
 *
 */
public class QueryUtil {

	protected static Logger logger = Logger.getLogger(QueryUtil.class);

	public static SearchByQueryParameter getQueryResultType(String query) throws SearchServiceException
	{
		try {
			SPQLLexer lexer = new SPQLLexer(new ANTLRStringStream(query));
			CommonTokenStream tokens = new CommonTokenStream(lexer);
			SPQLParser parser = new SPQLParser(tokens);
			ParsingResult result = parser.query();
			


			Map<SearchType, List<String>> terms = new HashMap<SearchType, List<String>>();
			
			for (TermType key : result.getTerms().keySet()) {
				
				if(key.equals(TermType.COMMON_NAME)){
					terms.put(SearchType.BY_COMMON_NAME, terms.get(key));
				}else if(key.equals(TermType.SCIENTIFIC_NAME)){
					terms.put(SearchType.BY_COMMON_NAME, terms.get(key));
				}
				
			}
			
			ReturnType type = result.getReturnExpression().getType();
			
			SearchResultType searchResultType = null;
			
			if (type == null) searchResultType = SearchResultType.SPECIES_PRODUCT;
			
			switch (type) {
				case ALL:{ 
					searchResultType = SearchResultType.SPECIES_PRODUCT;
					break;
				}
				case OCCURRENCE:{ 
					searchResultType = SearchResultType.OCCURRENCE_POINT;
					break;
				}
				case TAXON: {
					searchResultType = SearchResultType.TAXONOMY_ITEM;
					break;
				}
				default: {
					logger.error("Unknow return type: "+type + " setting default "+SearchResultType.SPECIES_PRODUCT);
					searchResultType = SearchResultType.SPECIES_PRODUCT;
				}
			}
			
			return new SearchByQueryParameter(terms, searchResultType);

		} catch (RecognitionException e) {
			logger.warn("Error parsing the user query", e);
			throw new SearchServiceException("Wrong query: "+e.getMessage());
		}
	}
	
	public static SearchResultType getResultType(SearchFilters searchFilters)
	{
		
		switch (searchFilters.getResultType()) {
			case RESULTITEM: return SearchResultType.SPECIES_PRODUCT;
			case TAXONOMYITEM: return SearchResultType.TAXONOMY_ITEM;
			default: logger.error("Unknow return type: "+searchFilters.getResultType());
		}
		return null;
	}
	
	public static SpeciesCapability getResultType(SearchResultType resultType)
	{
		
		switch (resultType) {
			case SPECIES_PRODUCT: return SpeciesCapability.RESULTITEM;
			case OCCURRENCE_POINT: return SpeciesCapability.OCCURRENCESPOINTS;
			case TAXONOMY_ITEM: return SpeciesCapability.TAXONOMYITEM;
		}
		return null;
	}
}
